/*
 * noisetest.c
 *
 * Copyright (C) 2001 Commonwealth Scientific and Industrial Research
 * Organisation (CSIRO), Australia.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.
 *
 */

#include <stdlib.h>
#include <mash.h>

int
main (int argc, char ** argv)
{
  MASH * mash;
  MashDeck * deck;
  MashTrack * track;
  MashLayer * l1, * l2;
  MashPlugin * noise_plugin;
  MashBase * noise1, * noise2;
  MashCount length;
  MashMonitor * monitor;
  int i;

  mash = mash_init ();
  mash_set_channels (mash, MASH_MONO);

  deck = mash_deck_new (mash);
  track = mash_add_track (mash, deck);

  noise_plugin = mash_find_plugin (mash, "mashstd::noise");

  noise1 = mash_new (mash, noise_plugin, NULL);
  noise2 = mash_new (mash, noise_plugin, NULL);

  l1 = mash_add_layer_ontop (mash, track, MASH_TIME_SAMPLES);
  for (i=0; i < 10; i++) {
    mash_add_sound (mash, noise1, l1, MASH_SAMPLES(i*2500),
		    MASH_SAMPLES(1250));
  }

  length = mash_length (mash, deck);

  monitor = mash_monitor_new (mash);

  l2 = mash_add_layer_ontop (mash, track, MASH_TIME_SAMPLES);
  mash_add_sound (mash, monitor, l2, MASH_SAMPLES(0), MASH_SAMPLES(length));

  mash_process (mash, deck, length, MashNone, MashNone);

  mash_purge (mash);

  exit (0);
}
