/*
 * sndfiletest.c
 *
 * Copyright (C) 2001 Commonwealth Scientific and Industrial Research
 * Organisation (CSIRO), Australia.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.
 *
 */

#include <stdlib.h>
#include <stdio.h>

#include <mash.h>

int
main (int argc, char ** argv)
{
  MASH * mash;
  MashDeck * deck, * deck2;
  MashTrack * track, * track2, * track3;
  MashLayer * l1, * l2, * l3, * l4, * l5;
  MashBase * sf1, * sf2;
  MashSquareTone * square;
  MashCount length;
  MashMonitor * monitor;
  MashSound * sm;
  MashEnvelope * env;
  MashPlugin * gain_plugin;
  MashPlugin * sf_plugin;
  MashBase * gain;
  CDSet * sf_parms;
  int i;
  int gain_key;
  int sf_path_key;

  mash = mash_init ();
  mash_set_tempo (mash, 140);
  mash_set_channels (mash, MASH_STEREO);

#if 0
  sf1 = mash_sndfile_new (mash, "./1052.wav", 0);
  sf2 = mash_sndfile_new (mash, "./909_cl.wav", 0);
#else
  sf_plugin = mash_find_plugin (mash, "builtin::sndfile_reader");
  sf_parms = cd_set_new (mash);
  sf_path_key = mash_get_init_parameter_key (mash, sf_plugin, "path");
  sf_parms = cd_set_insert (mash, sf_parms, sf_path_key,
		            CD_STRING("./1052.wav"));
  if (sf_plugin == NULL) {
    fprintf (stderr, "sf_plugin == NULL\n");
    exit (1);
  }
  sf1 = mash_new (mash, sf_plugin, sf_parms);
  sf_parms = cd_set_replace (mash, sf_parms, 1, CD_STRING("909_cl.wav"));
  sf2 = mash_new (mash, sf_plugin, sf_parms);
#endif
  square = mash_squaretone_new (mash, 220);
  monitor = mash_monitor_new (mash);

#if 0
  gain = mash_gain_new (mash);
#else
  gain_plugin = mash_find_plugin (mash, "builtin::gain");
  gain = mash_new (mash, gain_plugin, NULL);
#endif

  env = mash_envelope_new (mash, MASH_ENVELOPE_LINEAR);
  mash_envelope_set_timetype (mash, env, MASH_TIME_BEAT24S);
  mash_envelope_add_point (mash, env, MASH_BEAT24S(0), 0.1);
  mash_envelope_add_point (mash, env, MASH_BEAT24S(48), 1.3);
  mash_envelope_add_point (mash, env, MASH_BEAT24S(96), 0.1);

#if 0
  mash_gain_set_envelope (mash, gain, env);
#else
  gain_key = mash_get_parameter_key (mash, gain, "Gain envelope");
  printf ("got gain key: %d\n", gain_key);
  mash_set_parameter (mash, gain, gain_key, (CDScalar)((void *)env));
#endif

  deck = mash_deck_new (mash);
  track = mash_add_track (mash, deck);

  l1 = mash_add_layer_ontop (mash, track, MASH_TIME_BEAT24S);
  for (i=0; i < 4; i++) {
    mash_add_sound (mash, (MashBase *)sf1, l1, MASH_BEAT24S(i*24), MASH_BEAT24S(18));
  }

  track2 = mash_add_track (mash, deck);
  l2 = mash_add_layer_ontop (mash, track2, MASH_TIME_BEAT24S);
  for (i=0; i < 16; i++) {
    mash_add_sound (mash, (MashBase *)sf2, l2, MASH_BEAT24S(i*6), MASH_BEAT24S(4));
  }

  deck2 = mash_deck_new (mash);
  track3 = mash_add_track (mash, deck2);

  l3 = mash_add_layer_ontop (mash, track3, MASH_TIME_BEAT24S);
  mash_add_sound (mash, (MashBase *)deck, l3, MASH_BEAT24S(0), MASH_BEAT24S(96));

#if 1
  l4 = mash_add_layer_ontop (mash, track3, MASH_TIME_BEAT24S);
  mash_add_sound (mash, (MashBase *)gain, l4, MASH_BEAT24S(0), MASH_BEAT24S(96));
#endif

  l5 = mash_add_layer_ontop (mash, track3, MASH_TIME_SAMPLES);

  length = mash_length (mash, (MashBase *)deck);
  sm = mash_add_sound (mash, monitor, l5, MASH_SAMPLES(0), MASH_SAMPLES(length));

  printf ("deck: %p\ttrack: %p\tl1: %p\nl3: %p\tmonitor: %p\n",
          deck, track, l1, l3, monitor);

  mash_process (mash, deck2, length, MashNone, MashNone);

  mash_set_tempo (mash, 105);
  length = mash_length (mash, (MashBase *)deck);
  mash_sound_set_duration (mash, sm, MASH_SAMPLES(length));
  mash_seek (mash, deck2, 0, SEEK_SET);
  mash_process (mash, deck2, length, MashNone, MashNone);

  mash_set_tempo (mash, 168);
  length = mash_length (mash, (MashBase *)deck);
  mash_sound_set_duration (mash, sm, MASH_SAMPLES(length));
  mash_seek (mash, deck2, 0, SEEK_SET);
  mash_process (mash, deck2, length, MashNone, MashNone);

  mash_purge (mash);

  exit (0);
}
