/*
 * squaretest.c
 *
 * Copyright (C) 2001 Commonwealth Scientific and Industrial Research
 * Organisation (CSIRO), Australia.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <mash.h>

#define mash_add_sound_B(a,s,l,t1,t2) \
  mash_add_sound ((a), (s), (l), MASH_BEAT24S(t1), MASH_BEAT24S(t2))

#define BEAT 24
#define HALF_BEAT 12
#define QUARTER_BEAT 6
#define EIGHTH_BEAT 3

int
main (int argc, char ** argv)
{
  MASH * mash;
  MashDeck * deck, * deck2;
  MashTrack * track, * track2, * track3, * track4;
  MashLayer * l1, * l2, * l3, * l4, * l5;
  MashSound * s;
  MashSquareTone * square1, * square2, * square3, * square4, * square5;
  MashPlugin * noise_plugin;
  MashBase * noise;
  MashEnvelope * env1, * env2, * env3;
  MashEnvelope * be1, * be2, * be3;
  MashCount length;
  MashMonitor * monitor;
  int b = 0, b2 = 0, mb;

  mash = mash_init ();

  mash_set_tempo (mash, 160);
  mash_set_channels (mash, MASH_STEREO);

  square1 = mash_squaretone_new (mash, 220.0);
  square2 = mash_squaretone_new (mash, 440.0);
  square3 = mash_squaretone_new (mash, 385.0);
  square4 = mash_squaretone_new (mash, 231.0);
  square5 = mash_squaretone_new (mash, 165.0);

  noise_plugin = mash_find_plugin (mash, "builtin::noise");
  noise = mash_new (mash, noise_plugin, NULL);

  monitor = mash_monitor_new (mash);

  deck = mash_deck_new (mash);

#if 1
  track = mash_add_track (mash, deck);

  l1 = mash_add_layer_ontop (mash, track, MASH_TIME_BEAT24S);
  mash_add_sound_B (mash, square1, l1, b, BEAT); b += BEAT;
#if 1
  mash_add_sound_B (mash, square2, l1, b, QUARTER_BEAT); b += HALF_BEAT;
  mash_add_sound_B (mash, square2, l1, b, HALF_BEAT); b += HALF_BEAT;
  mash_add_sound_B (mash, square1, l1, b, HALF_BEAT); b += HALF_BEAT;
  mash_add_sound_B (mash, square2, l1, b, QUARTER_BEAT); b += HALF_BEAT;
  mash_add_sound_B (mash, square3, l1, b, HALF_BEAT); b += HALF_BEAT;
  mash_add_sound_B (mash, square2, l1, b, HALF_BEAT); b += HALF_BEAT;
#endif
#endif

#if 1
#if 1
  track2 = mash_add_track (mash, deck);
  mash_track_set_gain (mash, track2, 0.6);
#else
  track2 = track;
#endif
  l2 = mash_add_layer_ontop (mash, track2, MASH_TIME_BEAT24S);
  s = mash_add_sound_B (mash, square4, l2, b2, HALF_BEAT); b2 += BEAT;

  be1 = mash_envelope_new (mash, MASH_ENVELOPE_LINEAR);
  mash_envelope_set_timetype (mash, be1, MASH_TIME_BEAT24S);
  mash_envelope_add_point (mash, be1, MASH_BEAT24S(0), 0.9);
  mash_sound_set_blend_envelope (mash, s, be1);

#if 1
  s = mash_add_sound_B (mash, square5, l2, b2, HALF_BEAT); b2 += BEAT;
  be2 = mash_envelope_new (mash, MASH_ENVELOPE_LINEAR);
  mash_envelope_set_timetype (mash, be2, MASH_TIME_BEAT24S);
  mash_envelope_add_point (mash, be2, MASH_BEAT24S(0), 0.6);
  mash_sound_set_blend_envelope (mash, s, be2);

  s = mash_add_sound_B (mash, square4, l2, b2, BEAT); b2 += 48;
  be3 = mash_envelope_new (mash, MASH_ENVELOPE_LINEAR);
  mash_envelope_set_timetype (mash, be3, MASH_TIME_BEAT24S);
  mash_envelope_add_point (mash, be3, MASH_BEAT24S(0), 0.8);
  mash_sound_set_blend_envelope (mash, s, be3);
#endif
#endif

#if 1
  track4 = mash_add_track (mash, deck);
  mash_track_set_gain (mash, track4, 0.1);
  l5 = mash_add_layer_ontop (mash, track4, MASH_TIME_BEAT24S);
  mash_add_sound_B (mash, noise, l5, 0, b);
#endif

  mb = MAX (b, b2);

  length = mash_length (mash, deck);
  printf ("deck has length %ld\n", length);

  deck2 = mash_deck_new (mash);
  track3 = mash_add_track (mash, deck2);
  l3 = mash_add_layer_ontop (mash, track3, MASH_TIME_BEAT24S);

  s = mash_add_sound_B (mash, deck, l3, 0, mb);

#if 1
  env1 = mash_envelope_new (mash, MASH_ENVELOPE_LINEAR);
  mash_envelope_set_timetype (mash, env1, MASH_TIME_BEAT24S);
  mash_envelope_add_point (mash, env1, MASH_BEAT24S(0), 0.0);
  mash_envelope_add_point (mash, env1, MASH_BEAT24S(mb), 0.9);
  mash_sound_set_gain_envelope (mash, s, env1);
#endif

  s = mash_add_sound_B (mash, deck, l3, mb, mb);

#if 1
  env2 = mash_envelope_new (mash, MASH_ENVELOPE_LINEAR);
  mash_envelope_set_timetype (mash, env2, MASH_TIME_BEAT24S);
  mash_envelope_add_point (mash, env2, MASH_BEAT24S(0), 0.9);
  mash_sound_set_gain_envelope (mash, s, env2);
#endif

  s = mash_add_sound_B (mash, deck, l3, 2*mb, mb);

#if 1
  env3 = mash_envelope_new (mash, MASH_ENVELOPE_LINEAR);
  mash_envelope_set_timetype (mash, env3, MASH_TIME_BEAT24S);
  mash_envelope_add_point (mash, env3, MASH_BEAT24S(0), 0.9);
  mash_envelope_add_point (mash, env3, MASH_BEAT24S(mb), 0.0);
  mash_sound_set_gain_envelope (mash, s, env3);
#endif

  length = mash_length (mash, deck2);

  l4 = mash_add_layer_ontop (mash, track3, MASH_TIME_SAMPLES);
  mash_add_sound (mash, monitor, l4, MASH_SAMPLES(0), MASH_SAMPLES(length));

  printf ("deck: %p\ttrack: %p\tl1: %p\ttrack2: %p\tl2: %p\n",
          deck, track, l1, track2, l2);
  printf ("deck2: %p\ttrack3: %p\tl3: %p\tl4: %p\n", deck2, track3, l3, l4);
  printf ("square1: %p\tsquare2: %p\tsquare3: %p\nsquare4: %p\tsquare5: %p\n",
          square1, square2, square3, square4, square5);
  printf ("monitor: %p\n", monitor);

  mash_process (mash, deck2, length, MashNone, MashNone);

  mash_purge (mash);

  exit (0);
}
