/*
 * MASH -- An audio mixing and sequencing library.
 *
 * Copyright (C) 2001 Commonwealth Scientific and Industrial Research
 * Organisation (CSIRO), Australia.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * MashMetaText: Metadata for MashBases.
 *
 * Conrad Parker <Conrad.Parker@CSIRO.AU>, August 2001
 */


#define __MASH__
#include "mash.h"

#include <string.h>

MashMetaText *
mash_meta_text_new (MASH * mash)
{
  MashMetaText * mt;

  mt = (MashMetaText *) mash_malloc (sizeof (struct _MashMetaText));
  mt->authors = cd_list_new (mash);

  return mt;
}

char *
mash_meta_text_get_identifier (MASH * mash, MashMetaText * mt)
{
  return mt->identifier;
}

char *
mash_meta_text_set_identifier (MASH * mash, MashMetaText * mt, char * identifier)
{
  char * old = mt->identifier;
  mt->identifier = (identifier == NULL) ? NULL : strdup(identifier);
  return old;
}

char *
mash_meta_text_get_category (MASH * mash, MashMetaText * mt)
{
  return mt->category;
}

char *
mash_meta_text_set_category (MASH * mash, MashMetaText * mt, char * category)
{
  char * old = mt->category;
  mt->category = (category == NULL) ? NULL : strdup(category);
  return old;
}

char *
mash_meta_text_get_description (MASH * mash, MashMetaText * mt)
{
  return mt->description;
}

char *
mash_meta_text_set_description (MASH * mash, MashMetaText * mt, char * description)
{
  char * old = mt->description;
  mt->description = (description == NULL) ? NULL : strdup(description);
  return old;
}

char *
mash_meta_text_get_copyright (MASH * mash, MashMetaText * mt)
{
  return mt->copyright;
}

char *
mash_meta_text_set_copyright (MASH * mash, MashMetaText * mt, char * copyright)
{
  char * old = mt->copyright;
  mt->copyright = (copyright == NULL) ? NULL : strdup(copyright);
  return old;
}

char *
mash_meta_text_get_url (MASH * mash, MashMetaText * mt)
{
  return mt->url;
}

char *
mash_meta_text_set_url (MASH * mash, MashMetaText * mt, char * url)
{
  char * old = mt->url;
  mt->url = (url == NULL) ? NULL : strdup(url);
  return old;
}

static MashMetaAuthor *
mash_meta_author_new (MASH * mash, char * name, char * email)
{
  MashMetaAuthor * ma = mash_malloc (sizeof (struct _MashMetaAuthor));
  ma->name = (name == NULL) ? NULL : strdup(name);
  ma->email = (email == NULL) ? NULL: strdup(email);
  return ma;
}

CDList *
mash_meta_text_get_authors (MASH * mash, MashMetaText * mt)
{
  return mt->authors;
}
 
void
mash_meta_text_add_author (MASH * mash, MashMetaText * mt,
			 char * name, char * email)
{
  MashMetaAuthor * ma = mash_meta_author_new (mash, name, email);
  mt->authors = cd_list_append (mash, mt->authors, CD_POINTER(ma));
}

void
mash_meta_text_free (MASH * mash, MashMetaText * mt)
{
  cd_list_free_all (mash, mt->authors);
  mash_free (mt);
}
